<?php

App::uses('BuzzCustomersAppModel', 'BuzzCustomers.Model');

class CustomerAddress extends BuzzCustomersAppModel {

/**
 * Belongs to associations
 *
 * @var array
 */
	public $belongsTo = array(
		'Country',
		'UsState',
	);

/**
 * Validation rules
 *
 * @var array
 */
	public $validate = array(
		'first_name' => array(
			'required' => array(
				'rule' => 'notEmpty',
				'message' => 'Required'
			),
			'maxLength' => array(
				'rule' => array('maxLength', 45),
				'message' => 'No more than 45 characters long'
			)
		),
		'last_name' => array(
			'required' => array(
				'rule' => 'notEmpty',
				'message' => 'Required'
			),
			'maxLength' => array(
				'rule' => array('maxLength', 45),
				'message' => 'No more than 45 characters long'
			)
		),
		'address_line_1' => array(
			'required' => array(
				'rule' => 'notEmpty',
				'message' => 'Required'
			),
			'maxLength' => array(
				'rule' => array('maxLength', 45),
				'message' => 'No more than 45 characters long'
			)
		),
		'address_line_2' => array(
			'maxLength' => array(
				'rule' => array('maxLength', 45),
				'message' => 'No more than 45 characters long',
				'allowEmpty' => true
			)
		),
		'city' => array(
			'required' => array(
				'rule' => 'notEmpty',
				'message' => 'Required'
			),
			'maxLength' => array(
				'rule' => array('maxLength', 45),
				'message' => 'No more than 45 characters long'
			)
		),
		'county' => array(
			'maxLength' => array(
				'rule' => array('maxLength', 45),
				'message' => 'No more than 45 characters long',
				'allowEmpty' => true
			)
		),
		'country_id' => array(
			'required' => array(
				'rule' => 'notEmpty',
				'message' => 'Required'
			)
		),
		'us_state_id' => array(
			'required' => array(
				'rule' => array('validateRequiredByCountry', 226),
				'message' => 'Required'
			)
		),
		'postcode' => array(
			'required' => array(
				'rule' => 'notEmpty',
				'message' => 'Required'
			),
			'postcode' => array(
				'rule' => 'validatePostcode',
				'message' => 'Not a valid postcode'
			),
			'maxLength' => array(
				'rule' => array('maxLength', 20),
				'message' => 'No more than 20 characters long'
			)
		),
		'telephone' => array(
			'required' => array(
				'rule' => 'notEmpty',
				'message' => 'Required'
			),
			'telephone' => array(
				'rule' => '/^\+?[\d\ ]+$/',
				'message' => 'Not a valid telephone number'
			),
			'maxLength' => array(
				'rule' => array('maxLength', 20),
				'message' => 'No more than 20 characters long'
			)
		),
		'email' => array(
			'required' => array(
				'rule' => 'notEmpty',
				'message' => 'Required'
			),
			'email' => array(
				'rule' => 'email',
				'message' => 'Not a valid email address'
			),
			'maxLength' => array(
				'rule' => array('maxLength', 254),
				'message' => 'No more than 254 characters long'
			)
		)
	);

/**
 * Custom validation method for checking a postcode.
 *
 * @param array $data
 * @return bool
 */
	public function validatePostcode(array $data) {
		$check = array_pop($data);

		if (!empty($this->data[$this->alias]['country_id'])) {

			if ((int)$this->data[$this->alias]['country_id'] === 225) {
				// Check the postcode is valid in the UK.
				return preg_match('|^\D{1,2}\d{1,2} \d\D{2}$|i', $check) === 1;
			} elseif (!in_array($this->data[$this->alias]['country_id'], [81, 103])) {
				// Check a postcode has been supplied for all other countries
				// except Ghana and Ireland (as they don't use them).
				return !empty($check);
			}

		}

		return true;
	}

/**
 * Custom validation method for checking a field has been provided for a
 * particular country.
 *
 * @param array $data
 * @param int $countryId
 * @return bool
 */
	public function validateRequiredByCountry(array $data, $countryId) {
		$check = array_pop($data);

		if (!empty($this->data[$this->alias]['country_id'])) {

			if ((int)$this->data[$this->alias]['country_id'] === (int)$countryId) {
				// Check required field has been supplied.
				return !empty($check);
			}

		}

		return true;
	}

	public function beforeSave($options = []) {
		if (parent::beforeSave($options) === true) {

			// For non-US addresses we don't want to save a state.
			if (!empty($this->data[$this->alias]['country_id']) !== 226) {
				$this->data[$this->alias]['us_state_id'] = null;
			}

			return true;
		}
		return false;
	}

/**
 * Returns the default country ID to use for addresses.
 *
 * @return int
 */
	public static function getDefaultCountry() {
		if (Configure::check('BuzzCustomers.default_country')) {
			return Configure::read('BuzzCustomers.default_country');
		} else {
			// Fallback default if config not setup. Use the UK.
			return 225;
		}
	}

/**
 * Returns the default US state ID to use for addresses.
 *
 * @return int
 */
	public static function getDefaultUsState() {
		if (Configure::check('BuzzCustomers.default_state')) {
			return Configure::read('BuzzCustomers.default_state');
		} else {
			return;
		}
	}

}
